/* 
 * see http://bugzilla.suse.de/show_bug.cgi?id=31246
 *
 *
 * Initial Comment From Thorsten Kukuk 2003-09-18 09:51
 * %setup -q -n mod_perl-%{version}
 * gcc -o SOL_IPV6-test %{S:1}
 * if ! ./SOL_IPV6-test; then
 * %patch -p1
 * fi
 * 
 * As result, we will never now how the module was build, since this check depends
 * on the used kernel on the build machine. But this could be a 2.2.x kernel without
 * IPv6 support at all...
 * 
 * * How to reproduce:
 * 
 * Compile package on different hosts with different kernels and look,
 * how often the patch is applied and how often not.
 * 
 * Additional Comment #2 From Thorsten Kukuk 2003-09-18 10:12
 * Please document why this hack is necessary, or even better add a config-dist.sh
 * script, which tests for IPv6. We have enough build hosts with IPv6 and so we can
 * make sure, that it works because the test suite passes.
 * 
 * Additional Comment #3 From Thorsten Kukuk 2003-09-18 11:01
 * Hm, if you say on irc, it doesn't make a difference for the
 * tests, if the patch is applied or not, and that the only
 * difference is, that the unpatched version does not work on
 * a IPv4 only host, why not always applying this
 * patch and don't use config-dist.sh or any other wrapper?
 * It is much cleaner and more understandable for others.
 * 
 * Additional Comment #4 From Peter Poeml 2003-09-18 11:55
 * Testing a package that is built for 9.0 might sometimes be difficult on
 * a host that runs 7.2 and e.g. 2.2 kernel... So I apply a simple patch to
 * the configuration to still run the test.
 * 
 * Using a config-dist.sh to select hosts that support IPv6 only listening
 * sockets was what I did before. With that, we had barely enough build
 * hosts (gcc had to be installed for the test program), so sometimes the
 * package kept rebuilding for quite a while. It was pretty much okay on
 * i386 and x86_64, though.
 * 
 * I propose to leave it as is, for these reasons:
 *  1) the config-dist.sh does exactly the same test as I do now during
 *     building
 * 
 *  2) I'm patching the configuration only. And only for the testsuite.
 * 
 *  3) the real difference is in the apr lib; either apache supports
 *  
 * setsockopt(42, SOL_IPV6, 26, [0], 4)    = -1 ENOPROTOOPT (Protocol not available)
 * write(2, "[Mon Aug 04 14:03:46 2003] [crit"..., 130[Mon Aug 04 14:03:46 2003] [crit] \
 * (92)Protocol not available: make_sock: for address [::]:8543, apr_socket_opt_set: \
 * (IPV6_V6ONLY) ) = 130
 * close(42)                               = 0
 * write(2, "no listening sockets available, "..., 46no listening sockets available, \
 * shutting down ) = 46
 * 
 *     It is has nothing to do with mod_perl!
 * 
 *     - kernel must support Ipv6-only listening sockets. Sometimes this is
 *       not the case, I don't know when. My test program determines this.
 * 
 *     - apache tries to setsockopt SOL_IPV6 on the listening socket, if
 *       configured with "Listen <port>" _and_ Ipv6 is seems to be the
 *       default (probably influenced by IPv4-mapped IPv6 addresses)
 *       Simple workaround: using the "Listen <address>:<port>" syntax
 * 
 *  4) I have invested considerable time to find the reason for 3).  I have
 *     used config-dist.sh in autobuild to collect data like kernel
 *     versions, glibc, ipv4/ipv6 network configuration and so on -- to no
 *     avail. I have discussed with the mod_perl developers. It may be
 *     related to IPv4-mapped addresses.
 * 
 * > As result, we will never now how the module was build, since this check depends
 * > on the used kernel on the build machine. But this could be a 2.2.x kernel without
 * > IPv6 support at all...
 * 
 *  3) The build is exactly the same. It is the apache configuration that is
 *     patched to be able to run the test with Ipv4. The build of mod_perl
 *     is not affected, as detailed above.
 * 
 * I have tried to find the best solution for us. Any help with finding a
 * better solution is welcome.
 * 
 * 
 * Additional Comment #5 From Peter Poeml 2003-09-18 11:56
 * Created an attachment (id=6361)
 * the config-dist.sh that I used, for reference
 * 
 * Additional Comment #6 From Peter Poeml 2003-09-18 12:00
 * > a IPv4 only host, why not always applying this
 * > patch and don't use config-dist.sh or any other wrapper?
 * 
 * That's both possible. Always applying is not so good IMO, because then
 * the test would never run with IPv6. Who knows which bugs that could
 * trigger. Going back to config-dist.sh would be possible, but would it
 * garuantee that the package would finally work on all user's machines?
 * 
 * The way I do it now I _know_ that the package works on every host, it
 * only might need a tiny configuration change. With config-dist.sh the
 * testsuite would only run on those "special" hosts.
 * 
 */

#include <errno.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>

int main(void)
{
        int listenfd;
        int on = 0;

        listenfd = socket(AF_INET6, SOCK_STREAM, 0);

        /* hardcode IPV6_V6ONLY as 26 since it's not declared on most machines */
        if (setsockopt(listenfd, SOL_IPV6, 26, (void *)&on, sizeof(int)) == -1) {
                perror("setsockopt");
                return errno;
        }

        return 0;

}
